require 'active_model'
require 'active_model/attribute_methods'
require 'active_model/dirty'
require 'active_model/validations'
require 'active_model/conversion'
require 'active_model/naming'
require 'active_model/callbacks'

require 'quandl/cassandra/base/attributes'
require 'quandl/cassandra/base/callbacks'
require 'quandl/cassandra/base/connection'
require 'quandl/cassandra/base/naming'
require 'quandl/cassandra/base/persistence'
require 'quandl/cassandra/base/logging'
require 'quandl/cassandra/base/sanitization'
require 'quandl/cassandra/base/schema'
require 'quandl/cassandra/base/scoping'

class Quandl::Cassandra::Base
  
  include Quandl::Cassandra::Base::Connection
  include Quandl::Cassandra::Base::Logging if defined?(QUANDL_LOGGER) && QUANDL_LOGGER == true
  
  class << self
    
    def inherited(subclass)
      # remember models that inherit from base
      models << subclass unless models.include?(subclass)
      # include model behaviour
      subclass.class_eval do
        
        include ActiveModel::Validations

        include Quandl::Cassandra::Base::Scoping
        include Quandl::Cassandra::Base::Naming
        include Quandl::Cassandra::Base::Attributes
        include Quandl::Cassandra::Base::Persistence
        include Quandl::Cassandra::Base::Sanitization
        include Quandl::Cassandra::Base::Schema
        include Quandl::Cassandra::Base::Callbacks
        
      end
    end
    
    def models
      @@models ||= []
    end
    
  end
  
end