module Quandl
  module Cassandra
    class Base

    module Attributes

      extend ActiveSupport::Concern

      included do
        include ActiveModel::AttributeMethods
        include ActiveModel::Dirty
      end

      module ClassMethods

        def define_attributes(*names)
          Array(names).each{|name| define_attribute(name) }
          Quandl::Logger.info("The attributes for #{self.to_s} are #{Array(names).join(', ')}")
        end

        def define_attribute(name)
          name = name.to_sym
          self.class_eval do
            define_attribute_methods [name.to_sym]
          end
          define_method(name) do
            read_attribute(name)
          end
          define_method("#{name}=") do |value|
            write_attribute(name, value)
          end
          define_method("#{name}?") do
            read_attribute(name).present?
          end
          # store an array of defined attriubte names
          attribute_names << name unless attribute_names.include?(name)
        end

        def attribute_names
          @attribute_names ||= []
        end

      end

      def initialize(*args)
        attrs = args.extract_options!
        @attributes = {}
        # result set attributes?
        result_set = attrs.delete(:_result_set) if attrs.has_key?(:_result_set)
        # assign
        if result_set.present? && result_set.is_a?(Hash)
          @attributes = result_set.symbolize_keys!
          self.new_record = false
        end
        # assign attributes
        self.assign_attributes(attrs)
      end

      def inspect
        "#{self.class.name} " + attributes.inspect
      end

      def assign_attributes(hash)
        hash.each do |k,v|
          send("#{k}=", v) if self.class.attribute_names.include?(k) && respond_to?("#{k}=")
        end
      end

      def attributes
        @attributes ||= self.class.attribute_names.inject({}){|m,k| m[k] ||= nil; m }
      end

      def attributes=(value)
        @attributes = value.symbolize_keys! if value.is_a?(Hash)
      end

      def reload
        return false unless primary_key?
        clear_changes!
        reload_attributes
        true
      end

      protected

      def write_attribute(attribute, value)
        self.send(:"#{attribute}_will_change!") if self.attributes[:"#{attribute}"] != value
        @attributes[:"#{attribute}"] = value
      end

      def read_attribute(attribute)
        @attributes[:"#{attribute}"]
      end

      def attribute_changed?(attribute)
        !changes[attribute.to_s].nil?
      end

      def clear_changes!
        @previously_changed = {}
        @changed_attributes.clear
      end

      def cycle_changes!
        @previously_changed = changes
        @changed_attributes.clear
      end

    end

    end
  end
end