module Quandl::Cassandra::Base::Persistence
  
  extend ActiveSupport::Concern
  
  module ClassMethods
    
    def autosave_changes(*args)
      @autosave_changes = (args.first == true) unless args.first.nil?
      @autosave_changes = true if @autosave_changes.nil?
      @autosave_changes
    end
    
    def find_or_build(id)
      find(id) || new( id: id )
    end
    
    def create(*args)
      r = self.new(*args)
      r.save
      r
    end
    
    def new_from_query_result(result)
      return nil if result.blank?
      self.new( _result_set: result )
    end
    
  end
  
  attr_accessor :new_record
  
  def save!
    save
  end
  
  def save
    return false if self.class.autosave_changes && !primary_key?
    run_callbacks :save do
      touch
      save_changes! if self.class.autosave_changes == true
      cycle_changes!
      self.new_record = false
      true
    end
  end
  
  def touch
    self.created_at = Time.now if respond_to?(:created_at=) && created_at.blank?
    self.updated_at = Time.now if respond_to?(:updated_at=)
  end
  
  def destroy
    return false if new_record?
    run_callbacks :destroy do
      self.class.delete(id)
    end
  end
  
  def delete
    return false if new_record?
    self.class.delete(id)
  end
  
  def update_attributes(values)
    values = self.class.sanitize_attributes(values)
    # add primary key parts to values
    self.class.primary_key.each do |attribute|
      values[attribute.to_s] = sanitized_attribute(attribute) unless values.has_key?(attribute.to_s)
    end
    # update
    self.class.execute("INSERT INTO #{self.class.table_name} (#{values.keys.join(', ')}) VALUES (#{values.values.join(', ')})")
  end
  
  def new_record?
    new_record
  end
  
  def new_record
    return @new_record if defined?(@new_record)
    @new_record = true
  end
  
  def new_record=(value)
    @new_record = (value == true)
  end
  
  private
  
  def reload_attributes
    self.attributes = self.class.where(primary_key_attributes).limit(1).to_h.first
  end
  
  def save_changes!
    attrs = {}
    changes.each{|k,v| attrs[k] = v[1] }
    update_attributes(attrs)
  end
  
end