module Quandl::Cassandra::Base::Sanitization
  extend ActiveSupport::Concern

  module ClassMethods
    def bind_primary_key(*args)
      attrs = {}
      primary_key.each_with_index { |k, i| attrs[k] = args[i] if args[i].present? }
      attrs
    end

    def sanitize_attributes(attrs)
      values = {}
      attrs.each do |attribute, value|
        # exclude attributes that are not present in columns
        next unless column_type(attribute)
        # sanitize value for cql
        values[attribute] = sanitize_attribute(attribute, value)
      end
      values
    end

    def sanitize_attribute(attribute, value)
      # process each value
      return value.map { |val| column_type(attribute).sanitize_for_cql(val) } if value.is_a?(Array)
      # process value
      column_type(attribute).sanitize_for_cql(value)
    end
  end

  def sanitized_attribute(attribute)
    self.class.sanitize_attribute(attribute, read_attribute(attribute))
  end
end
