class Quandl::Strategy
  
  attr_accessor :attributes
  
  def initialize(*args, &block)
    setup(*args, &block)
  end
  
  def setup(attributes, &block)
    self.attributes = attributes
    block.call(self) if block_given?
  end
  
  def perform
    classes.each do |klass|
      strategy = klass.new(attributes)
      self.attributes = strategy.attributes
    end
    attributes
  end
  
  def use(klass)
    self.classes << klass unless classes.include?(klass)
  end
  
  def classes
    @classes ||= []
  end
  
  class Strategize
      
    attr_accessor :attributes
    
    def self.call(attrs)
      self.new(attrs).attributes
    end
      
    def initialize(attrs)
      attrs = attrs.clone if attrs.respond_to?(:clone)
      self.attributes = attrs
      self.perform if respond_to?(:perform)
    end
    
    def self.define_attributes(*names)
      names.each do |name|
        # getter
        define_method(name) do
          self.attributes[name.to_sym]
        end
        # setter
        define_method("#{name}=") do |value|
          self.attributes[name.to_sym] = value
        end
        # present?
        define_method("#{name}?") do
          self.attributes[name.to_sym].present?
        end
      end
    end
      
  end
  
end
